#------------------------------------------------------------------
# $Copyright: Copyright (c) 2013 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Header: /project/isv-agent/VProAgents/SybaseIQ/SybaseIQ.pm,v 1.15 2013/11/29 06:11:09 kkorde Exp $
#
# $sAuthor: $
#
# $Date: 2013/11/29 06:11:09 $
# $Revision: 1.15 $
#
# Provides:                            SybaseIQ
# Author:                              kkorde
# crAgent Build:                       \$Id: crAgent,v 1.97 2011
# crAgent Run Date:                    09 Nov, 2011
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------

#------------
# Be strict..
#------------
use strict;

#----------------------
# Declare the package..
#----------------------
package SybaseIQ;

use ag_i18n_inc;

#-------------------------------------
# Include required framework modules..
#-------------------------------------
use Sys;            # Support generalized UNIX commands
use Proc;           # For calls to the process routines
use Comms;          # For calls to socket(3C) routines
use Version;        # Version display generator
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses

#--------------------------------
# Include standard PERL modules..
#--------------------------------
use Fcntl;
use Socket;
use File::Path qw ( mkpath );
use File::stat;

my ( $CLUS_HOME, $sPrefix ) = ();

#------------------------
# Setup VCS/VCSone home..
#------------------------
if ( $ENV { CLUSTER_HOME } ) { # VCS 5.x/VCSOne..
  $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcsone";
}
else { # VCS 4.x..
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /.*(vad|vcs(|one))$/;

#-----------------------------------------
# Define module-vendor specific commands..
#-----------------------------------------
my $sCmdHares  = $HARES  ||= "$CLUS_HOME/bin/hares";
my $sCmdHatype = $HATYPE ||= "$CLUS_HOME/bin/hatype";
my $sCmdVcsDecrypt = $HADECRYPT ||= "$CLUS_HOME/sbin/${sPrefix}decrypt";

my @liRunningProcs = (); # Global..

#-----------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested..
#-----------------------------------------------------------------------
my $sVer = "\$Id: SybaseIQ.pm,v 1.15 2013/11/29 06:11:09 kkorde Exp $::ETVHeader";
Version::TellVersion ( $sVer );

#--------------------------------
# Define SybaseIQ subroutines..
#--------------------------------

=pod

 -------------------------------------------------------------------------
 Function :  bool ArgsValid ( IN String EntryPointName,
                              IN String SybIQUser,
                              IN String EnvFile,
                              IN String SybIQBinDir,
                              IN String ServerName,
                              IN String StartOpts,
                              IN String SybIQDB,
                              IN String SybIQDBConfFile,
                              IN String SybIQDBAdmin,
                              IN String SybIQDBAdminPasswd,
                              IN Integer SecondLevelMonitor,
                              IN String MonitorProgram );

 Description : Validates the ArgList passed to the entry point.

 Parameters :

 $sEntryPointName               Entry pt name
 $localSybIQUser                Specifies the user name required to run SybaseIQ commands.
 $localEnvFile                  Absolute path to the Sybase IQ environment file that will be sourced.
 $localSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $localServerName               Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $localStartOpts                Additional "Sybase IQ" server options while starting the SybaseIQ server instance.
 $localSybIQDB                  Complete path of the Sybase IQ database that is queried during detailed monitoring.
 $localSybIQDBConfFile          Complete path of the Sybase IQ database configuration file.
 $localSybIQDBAdmin             A Sybase database administrator who would has permission to stop Sybase IQ server instance.This user should have query privileges to the table in provided database.
 $localSybIQDBAdminPasswd       SybIQDBAdmPassowrd is the Sybase database password for SybIQDBAdmin who would query the Sybase database for detailed monitoring.
 $localSecondLevelMonitor Cycle count to run in-depth monitoring.
 $localMonitorProgram           Full path to external monitor program.
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ArgsValid {
  #----------------------------------------
  # Set up local vars from passed in ones..
  #----------------------------------------
  my ( $sEntryPointName,
       $localSybIQUser,
       $localEnvFile,
       $localSybIQBinDir,
       $localServerName,
       $localStartOpts,
       $localSybIQDB,
       $localSybIQDBConfFile,
       $localSybIQDBAdmin,
       $localSybIQDBAdminPasswd,
       $localSecondLevelMonitor,
       $localMonitorProgram ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName           [%s]
  SybIQUser                [%s]
  EnvFile                  [%s]
  SybIQBinDir              [%s]
  ServerName               [%s]
  StartOpts                [%s]
  SybIQDB                  [%s]
  SybIQDBConfFile          [%s]
  SybIQDBAdmin             [%s]
  SybIQDBAdminPasswd       [XXXXX]
  SecondLevelMonitor       [%s]   
  MonitorProgram           [%s]",

  $sMyName,   $sEntryPointName,   $localSybIQUser,   $localEnvFile,
  $localSybIQBinDir,       $localServerName,       $localStartOpts,
  $localSybIQDB,     $localSybIQDBConfFile,     $localSybIQDBAdmin,
  $localSecondLevelMonitor, $localMonitorProgram );

  #-------------------------------------
  # Set return code to hopeful success..
  #-------------------------------------
  my $iReturnCode = 1;

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my ( @lsMissing, %hsArgList ) = ();
  my @lsArgs =
  qw ( EntryPointName SybIQUser EnvFile SybIQBinDir ServerName StartOpts
       SybIQDB    SybIQDBConfFile    SybIQDBAdmin     SybIQDBAdminPasswd
       SecondLevelMonitor MonitorProgram );

  @hsArgList { @lsArgs } = @_; # Assign..

  #----------------------------------------
  # Reqd. attributes based on entry point..
  #----------------------------------------

  my @lsOnlineArgs  = qw ( SybIQUser EnvFile SybIQBinDir ServerName );
  my @lsOfflineArgs  = qw ( SybIQUser EnvFile SybIQBinDir ServerName );

  # The SybaseIQ server instance may be stopped without the need of SybIQDBAdmin & SybIQDBAdminPasswd
  # when the SybaseIQ server is started without specifying the database.
  # If the SybaseIQ database is specified then we need to validate SybIQDBAdminPasswd & ServerName

  if ( $localSybIQDB ) { 
     @lsOfflineArgs = qw ( SybIQUser EnvFile SybIQBinDir SybIQDBAdmin SybIQDBAdminPasswd ServerName );
  }

  my @lsMonitorArgs = qw ( SybIQUser EnvFile SybIQBinDir ServerName );
  my @lsCleanArgs   = qw ( SybIQUser EnvFile SybIQBinDir ServerName );
  my @lsImfRegisterArgs = qw ( SybIQUser EnvFile SybIQBinDir ServerName );

  my %hlsNeeded = ( 'online'  => \@lsOnlineArgs,
                    'offline' => \@lsOfflineArgs,
                    'monitor' => \@lsMonitorArgs,
                    'clean'   => \@lsCleanArgs,
	            'imf_register'  => \@lsImfRegisterArgs );

  my @lsReqdArgs = @{ $hlsNeeded { $sEntryPointName } };

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20002,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return, if failed..

  #-----------------------------
  # Remove SLM file, if exists..
  #-----------------------------
  if ( $sEntryPointName ne 'monitor' ) {
    VCSagentFW::RunSecondLevelMonitor ( 1 ); # Delete..
  }

  #-------------------------------
  # Individual validation starts..
  #-------------------------------
  

  #----------------------------------
  # Validation for all entry points..
  #----------------------------------
  #-------------------------------
  # Validate parameter SybIQBinDir..
  #-------------------------------
  if ( $sEntryPointName eq "online"  ||
       $sEntryPointName eq "offline" ){

     if ( ! -d $localSybIQBinDir ) {
          &logError ( 20003,
          "<%s> SybIQBinDir [%s] directory does not exist",
          $sMyName, $localSybIQBinDir );
          $iReturnCode = 0; # Failure..
     }

     #---------------------------
     # Validate parameter EnvFile..
     #---------------------------
     if ( ! -f $localEnvFile ) {
          &logError ( 20004,
          "<%s> EnvFile [%s] is not a plain file",
          $sMyName, $localEnvFile );
          $iReturnCode = 0; # Failure..
     }
  }
   
  #-----------------------------
  # Validate parameter SybIQUser..
  #-----------------------------
  if ( ! defined getpwnam $localSybIQUser ) {
     &logError ( 20005,
      "<%s> SybIQUser [%s] is not a valid UNIX login",
     $sMyName, $localSybIQUser );
     $iReturnCode = 0; # Failure..
  }

  #-----------------------------
  # Validate parameter ServerName..
  #-----------------------------
  if(!($localServerName =~ /(\S+)/)){
     &logError ( 20006,
     "<%s> SybaseIQ server instance name has not been provided",
     $sMyName );
     $iReturnCode = 0; # Failure..
  }

  #---------------------------------------
  # Validation for online entry point(s)..
  #---------------------------------------
  if ( $sEntryPointName eq "online" ) {

    if($localSybIQDBConfFile =~ /(\S+)/){
       if(!($localSybIQDB =~ /(\S+)/)){
          &logError ( 20007,
          "<%s> SybaseIQ DB File has not been provided",
          $sMyName );
          $iReturnCode = 0; # Failure..
        }
    }

    #------------------------------------
    # Validate parameter MonitorProgram..
    #------------------------------------
    if ( $localMonitorProgram =~ /(\S+)/ && # Defined..
         ! -x $1 ) {
        &logError ( 20008,
        "<%s> MonitorProgram [%s] is not executable",
        $sMyName, $1 );
        $iReturnCode = 0; # Failure..
    } 
  }

  $iReturnCode ||
  &logWarn ( 20009,
  "<%s> Some of the attributes could not be validated. Returning [%s]",
  $sMyName, $iReturnCode );

  return $iReturnCode ;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Start ( IN String EnvFile,
                         IN Integer SecondLevelMonitor,
                         IN String ServerName,
                         IN String StartOpts,
                         IN String SybIQBinDir,
                         IN String SybIQDB,
                         IN String SybIQDBAdmin,
                         IN String SybIQDBAdminPasswd,
                         IN String SybIQDBConfFile,
                         IN String SybIQUser );

 Description : Start logic needed for online entry pt

 Parameters :

 $sEnvFile                  Absolute path to the Sybase IQ environment file that will be sourced.
 $iSecondLevelMonitor       Cycle count to run in-depth monitoring.
 $sServerName               Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $sStartOpts                Additional "Sybase IQ" server options while starting the SybaseIQ server instance.
 $sSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDB                  Complete path of the Sybase IQ database that is queried during detailed monitoring.
 $sSybIQDBAdmin             A Sybase database administrator who would has permission to stop Sybase IQ server instance.This user should have query privileges to the table in provided database.
 $sSybIQDBAdminPasswd       SybIQDBAdmPassowrd is the Sybase database password for SybIQDBAdmin who would query the Sybase database for detailed monitoring.
 $sSybIQDBConfFile          Complete path of the Sybase IQ database configuration file.
 $sSybIQUser                Specifies the user name required to run SybaseIQ commands.
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Start {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sEnvFile,
       $iSecondLevelMonitor,
       $sServerName,
       $sStartOpts,
       $sSybIQBinDir,
       $sSybIQDB,
       $sSybIQDBAdmin,
       $sSybIQDBAdminPasswd,
       $sSybIQDBConfFile,
       $sSybIQUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20010, "Subroutine <%s> called with:
  EnvFile                  [%s]
  SecondLevelMonitor       [%s]
  ServerName               [%s]
  StartOpts                [%s]
  SybIQBinDir              [%s]
  SybIQDB                  [%s]
  SybIQDBAdmin             [%s]
  SybIQDBAdminPasswd       [XXXXX]
  SybIQDBConfFile          [%s]
  SybIQUser                [%s]",

  $sMyName,    $sEnvFile, $iSecondLevelMonitor,   $sServerName,
  $sStartOpts,    $sSybIQBinDir,     $sSybIQDB,     $sSybIQDBAdmin,
  $sSybIQDBConfFile, $sSybIQUser );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sCommand, $sCommandArgs, $iTillWhen, $iExitCode, $iTimeout,
       @lsCmdout, @liPids ) = ();

  #-----------------------------------------
  # Whack any preexisting SybaseIQ processes..
  #-----------------------------------------
  &logTrace ( 20011,
  "<%s> Killing SybaseIQ processes for this instance", $sMyName );

  if ( SybaseIQ::KillSybaseIQProcesses ( $sServerName,
                                         $sStartOpts,
                                         $sSybIQBinDir,
                                         $sSybIQDB,
                                         $sSybIQDBConfFile,
                                         $sSybIQUser ) ) {
    &logTrace ( 20012,
    "<%s> Processes for this instance do not exist before start-up",
    $sMyName );
  }
  else {
    &logError ( 20013,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );
    $iReturnCode = 0; # Failure..
  }
  
  #--------------------
  # Return, if failed..
  #--------------------
  $iReturnCode || return $iReturnCode;

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 );
  
  #------------------------------------------
  # Define start command and args variables..
  #------------------------------------------
  $sCommand = "$sSybIQBinDir/start_iq";
  $sCommandArgs = "";
  ( @lsCmdout, $iExitCode, $iTimeout ) = ();

  
  # When user has provide the Sybase DB file 
  # Command would be start_iq $sStartOpts -n $ServerName @db_config_file dbfile

  if($sSybIQDB =~ /(\S+)/){
     if($sSybIQDBConfFile =~ /(\S+)/){
        $sCommandArgs = "$sStartOpts -n $sServerName \@$sSybIQDBConfFile $sSybIQDB";
     }else{  
        $sCommandArgs = "$sStartOpts -n $sServerName $sSybIQDB";
     }
  }else{  
     $sCommandArgs = "$sStartOpts -n $sServerName";
  }

  &logTrace ( 20014,
  "<%s> Starting SybaseIQ instance with Cmd [%s] Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  #------------------
  # Request startup..
  #------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20015,
    "<%s> Insufficient time to continue boot up. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = 0; # Failure..
  }
  elsif ( $iReturnCode =
    SybaseIQ::ExecLocalCmd ( $sSybIQUser, 
                             $sEnvFile, 
                             $sCommand, 
                             $sCommandArgs, 
                             \$iExitCode, 
                             $iTimeout, 
                             \@lsCmdout ) ) {
    &logTrace ( 20016,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgs );
  }

  #-------------------
  # Return if failed..
  #-------------------
  $iReturnCode || return $iReturnCode;


  #--------------------
  # Delay entry point..
  #--------------------
  my $iSleepTime = 10; # Secs..
  
  while ( ( $iTimeout = $iTillWhen - time ) > 10 ) {
    @liPids = (); # Initialise..

    sleep $iSleepTime; # Sec..

    &logTrace ( 20017,
    "<%s> Confirming start-up with [%s] sec in hand",
    $sMyName, $iTimeout - $iSleepTime );

    #---------
    # Do FLM..
    #---------
    if ( VCSDef_MonitorExit::ONLINE ==
         SybaseIQ::GetProcState ( \@liPids,
                                  $sServerName,
                                  $sStartOpts,
                                  $sSybIQBinDir,
                                  $sSybIQDB,
                                  $sSybIQDBConfFile,
                                  $sSybIQUser,
                                  1 ) && (           #Running FFLM..
         $iSecondLevelMonitor  ?                     # If enabled..
         VCSDef_MonitorExit::ONLINE ==               #Online after..
         SybaseIQ::SecondLevelMonitor ( $sEnvFile,   # Running SLM..
                                        $sServerName,
                                        $sSybIQBinDir,
                                        $sSybIQDBAdmin,
                                        $sSybIQDBAdminPasswd,
                                        $sSybIQDB,
                                        $sSybIQUser,
                                        $iTillWhen ) : 1 ) ) { 
      #---------------------
      # Start-up confirmed..
      #---------------------
      &logInfo ( 20018,
      "<%s> Confirmed start-up of this instance of SybaseIQ server",
      $sMyName );

      $iReturnCode = 1; # Success..
      last;
    }
    else {
      &logTrace ( 20019,
      "<%s> Could not confirm start-up. Sleeping for [%s] sec",
      $sMyName, $iSleepTime );

      $iReturnCode = 0; # Failure..
    }
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool Stop ( IN String EnvFile,
                        IN String ServerName,
                        IN String StartOpts,
                        IN String SybIQBinDir,
                        IN String SybIQDB,
                        IN String SybIQDBAdmin,
                        IN String SybIQDBAdminPasswd,
                        IN String SybIQDBConfFile,
                        IN String SybIQUser );

 Description : Stop logic needed for offline entry pt

 Parameters :

 $sEnvFile                  Absolute path to the Sybase IQ environment file that will be sourced.
 $sServerName               Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $sStartOpts                Additional "Sybase IQ" server options while starting the SybaseIQ server instance.
 $sSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDB                  Complete path of the Sybase IQ database that is queried during detailed monitoring.
 $sSybIQDBAdmin             A Sybase database administrator who would has permission to stop Sybase IQ server instance.This user should have query privileges to the table in provided database.
 $sSybIQDBAdminPasswd       SybIQDBAdmPassowrd is the Sybase database password for SybIQDBAdmin who would query the Sybase database for detailed monitoring.
 $sSybIQDBConfFile          Complete path of the Sybase IQ database configuration file.
 $sSybIQUser                Specifies the user name required to run SybaseIQ commands.
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Stop {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sEnvFile,
       $sServerName,
       $sStartOpts,
       $sSybIQBinDir,
       $sSybIQDB,
       $sSybIQDBAdmin,
       $sSybIQDBAdminPasswd,
       $sSybIQDBConfFile,
       $sSybIQUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20020, "Subroutine <%s> called with:
  EnvFile                  [%s]
  ServerName               [%s]
  StartOpts                [%s]
  SybIQBinDir              [%s]
  SybIQDB                  [%s]
  SybIQDBAdmin             [%s]
  SybIQDBAdminPasswd       [XXXXX]
  SybIQDBConfFile          [%s]
  SybIQUser                [%s]",

  $sMyName, $sEnvFile,  $sServerName,  $sStartOpts,  $sSybIQBinDir,
  $sSybIQDB,         $sSybIQDBAdmin,  $sSybIQDBConfFile, $sSybIQUser );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 ); # With margin..

  #------------------------------------------
  # Declare stop command and args variables..
  #------------------------------------------
  my ( $sCommand, $sCommandArgs, $iTimeout, $iExitCode, $iSleepTime,
       @lsCmdout, @liPids ) = ();
  
  #------------------------------------
  # Define stop command and arguments..
  #------------------------------------
  $sCommand = "$sSybIQBinDir/dbstop";
  $sCommandArgs = "";
  ( $iExitCode, $iTimeout, @lsCmdout ) = (); # Truncate.. 
  
  my ($sSwitch, $sPsCmd, $sPsCmdArgs, $sGrepCmd, $sexecCmd, @fields, $sGetCntCmd, $sIQSrvCnt, $sProcName) = ();
  $sIQSrvCnt = 0;
  $sSwitch = $^O;    # Get OS..
  $sProcName = "$sSybIQBinDir/iqsrv15";
  $sGrepCmd = "/bin/grep";

  if ( "linux" eq $sSwitch ) {

      $sPsCmd = "/bin/ps";
      $sPsCmdArgs = " -eo user,pid,args";
      $sPsCmd .= $sPsCmdArgs;

      if ( $sSybIQBinDir =~ /IQ-16_\d+\S*\/(bin|bin32|bin64)\b/ ){
         $sProcName = "$sSybIQBinDir/iqsrv16";
      }

      $sexecCmd = "$sPsCmd | $sGrepCmd $sProcName | $sGrepCmd $sServerName";
      
      # Below command would be fired to see how many iqsrv15/iqsrv16 processess 
      # running. Need to do this is because when only one iqsrv15/iqsrv16 server instance
      # is running with no database association we can use "stop_iq -stop one" command
      $sGetCntCmd = "$sPsCmd | $sGrepCmd $sProcName | $sGrepCmd -v grep";

  } elsif ( "solaris" eq $sSwitch ) {

      $sPsCmd = "/usr/ucb/ps";
      $sPsCmdArgs = " -auxwwl";
      $sPsCmd .= $sPsCmdArgs;

      if ( $sSybIQBinDir =~ /IQ-16_\d+\S*\/(bin|bin32|bin64)\b/ ){
         $sProcName = "$sSybIQBinDir/iqsrv16";
      }

      $sexecCmd = "$sPsCmd | $sGrepCmd $sProcName | $sGrepCmd $sServerName";
      $sGetCntCmd = "$sPsCmd | $sGrepCmd $sProcName | $sGrepCmd -v grep";
  }

  #------------------------------------
  # Define stop command and arguments..
  #------------------------------------
  $sCommand = "$sSybIQBinDir/dbstop";
  my $decryptedSybIQDBAdminPasswd = DecryptPassword($sSybIQDBAdminPasswd);
  my $sCommandArgsTemp = "";
  if($sSybIQDB){
     $sCommandArgs = " -c \"uid=$sSybIQDBAdmin;pwd=$decryptedSybIQDBAdminPasswd;eng=$sServerName;links=tcpip\"";
     $sCommandArgsTemp = " -c \"uid=$sSybIQDBAdmin;pwd=XXXXX;eng=$sServerName;links=tcpip\"";
     &logTrace ( 20021,
     "<%s> Stopping SybaseIQ server instance [%s] with Command [%s] Args [%s]",
     $sMyName, $sServerName, $sCommand, $sCommandArgsTemp );
  }else{

     open(PIDCNT,"$sGetCntCmd |");
     while(<PIDCNT>){
       $sIQSrvCnt++;
     }

     if($sIQSrvCnt > 1){
        open(PIDLIST,"$sexecCmd |");
        while(<PIDLIST>){
          chomp;
          @fields = split(/\s+/, $_);
      
          if(kill(0,$fields[1])){
             kill(15,$fields[1]);
             &logTrace ( 20022,
             "<%s> Stopping SybaseIQ server instance [%s].", $sMyName, $sServerName );
          }
          last;
        }
     }else{
        $sCommand = "$sSybIQBinDir/stop_iq";
        $sCommandArgs = " -stop one"; 
        &logTrace ( 20023,
        "<%s> Stopping SybaseIQ server instance [%s] with Command [%s] Args [%s]",
        $sMyName, $sServerName, $sCommand, $sCommandArgs );
     }
  }

  #------------------------
  # Request shutdown..
  #------------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20024,
    "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = 0; # Failure..
  }
  elsif ( $iReturnCode =
          SybaseIQ::ExecLocalCmd ( $sSybIQUser, 
                                   $sEnvFile, 
                                   $sCommand, 
                                   $sCommandArgs, 
                                   \$iExitCode, 
                                   $iTimeout, 
                                   \@lsCmdout ) ) {

    $sCommandArgsTemp = $sCommandArgs;
    if($sCommandArgsTemp =~ s/pwd=\w+/pwd=XXXX/){
       #do nothing continue
    }
    &logTrace ( 20025,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgsTemp );
  }

  #--------------------
  # Delay entry point..
  #--------------------
  if ( $iReturnCode ) { # Success..
    #-----------------
    # Set sleep time..
    #-----------------
    $iSleepTime = 10; # Sec..
  
    &logInfo ( 20026,
    "<%s> Delaying exit to confirm shutdown..", $sMyName );

    #--------------------
    # Delay entry point..
    #--------------------
    while ( ( $iTimeout = $iTillWhen - time ) > 20 ) {
      #-----------------------
      # Wait before retrying..
      #-----------------------
      sleep $iSleepTime; # Secs..
  
      SybaseIQ::GetProcState ( \@liPids,
                               $sServerName,
                               $sStartOpts,
                               $sSybIQBinDir,
                               $sSybIQDB,
                               $sSybIQDBConfFile,
                               $sSybIQUser,
                               0 ); # Request regular..
  
      #------------------------------
      # Check for pending processes..
      #------------------------------
      if ( scalar @liPids ) {
        &logTrace ( 20027,
        "<%s> Could not confirm shutdown. Retrying in [%s] sec.
        Delaying kill with [%s] sec in hand",
        $sMyName, $iSleepTime, $iTimeout - $iSleepTime );
      }
      else {
        &logInfo ( 20028,
        "<%s> Confirmed shutdown of SybaseIQ instance",
        $sMyName );
        last; # Get out..
      }
    }
  }

  #-----------------------------------------
  # Whack any persisting SybaseIQ processes..
  #-----------------------------------------
  &logTrace ( 20029,
  "<%s> Killing SybaseIQ processes for this instance", $sMyName );

  if ( SybaseIQ::KillSybaseIQProcesses ( $sServerName,
                                         $sStartOpts,
                                         $sSybIQBinDir,
                                         $sSybIQDB,
                                         $sSybIQDBConfFile,
                                         $sSybIQUser ) ) {
    &logTrace ( 20030,
    "<%s> Processes for this instance do not exist after shutdown",
    $sMyName );
  }
  else {
    &logError ( 20031,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );

    $iReturnCode = 0; # Failure..
  }
  
  return $iReturnCode;
}



=pod

 -------------------------------------------------------------------------
 Function : bool Kill ( IN Integer CleanReason,
                        IN String EnvFile,
                        IN String ServerName,
                        IN String StartOpts,
                        IN String SybIQBinDir,
                        IN String SybIQDB,
                        IN String SybIQDBAdmin,
                        IN String SybIQDBAdminPasswd,
                        IN String SybIQDBConfFile,
                        IN String SybIQUser );

 Description : Forces shutdown of resource needed for clean entry point

 Parameters :

 $iCleanReason              Why clean was called
 $sEnvFile                  Absolute path to the Sybase IQ environment file that will be sourced.
 $sServerName               Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $sStartOpts                Additional "Sybase IQ" server options while starting the SybaseIQ server instance.
 $sSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDB                  Complete path of the Sybase IQ database that is queried during detailed monitoring.
 $sSybIQDBAdmin             A Sybase database administrator who would has permission to stop Sybase IQ server instance.This user should have query privileges to the table in provided database.
 $sSybIQDBAdminPasswd       SybIQDBAdmPassowrd is the Sybase database password for SybIQDBAdmin who would query the Sybase database for detailed monitoring.
 $sSybIQDBConfFile          Complete path of the Sybase IQ database configuration file.
 $sSybIQUser                Specifies the user name required to run SybaseIQ commands.
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Kill {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iCleanReason,
       $sEnvFile,
       $sServerName,
       $sStartOpts,
       $sSybIQBinDir,
       $sSybIQDB,
       $sSybIQDBAdmin,
       $sSybIQDBAdminPasswd,
       $sSybIQDBConfFile,
       $sSybIQUser ) = @_;

  #-----------------------------
  # Figure out why were called..
  #-----------------------------
  my ( $sCleanReason, %hsCleanReasons ) = ();
  my @lsCleanStrings = qw ( Offline_timed_out    Offline_was_ineffective
                            Online_timed_out      Online_was_ineffective
                            Found_offline_unexpectedly Monitor_timed_out );

  @hsCleanReasons { 0 .. $#lsCleanStrings } = @lsCleanStrings; # Populate..
  ( $sCleanReason = $hsCleanReasons { $iCleanReason } ||       # Reason..
                    $iCleanReason ) =~ s/_/ /g;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20032, "Subroutine <%s> called with:
  CleanReason              [%s]
  EnvFile                  [%s]
  ServerName               [%s]
  StartOpts                [%s]
  SybIQBinDir              [%s]
  SybIQDB                  [%s]
  SybIQDBAdmin             [%s]
  SybIQDBAdminPasswd       [XXXXX]
  SybIQDBConfFile          [%s]
  SybIQUser                [%s]",

  $sMyName, $sCleanReason,  $sEnvFile,  $sServerName,  $sStartOpts,
  $sSybIQBinDir, $sSybIQDB,  $sSybIQDBAdmin,  
  $sSybIQDBConfFile, $sSybIQUser );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 ); # With margin..

  #--------------------------
  # Declare local variables..
  #--------------------------
  my ( $sCommand, $sCommandArgs, $iTimeout, $iExitCode,
       @lsFiles, @lsCmdout ) = ();
  
  #-------------------------------------
  # Define clean command and arguments..
  #-------------------------------------
  $sCommand = "$sSybIQBinDir/dbstop";
  $sCommandArgs = "";

  my ($sSwitch, $sPsCmd, $sPsCmdArgs, $sGrepCmd, $sexecCmd, @fields, $sGetCntCmd, $sIQSrvCnt, $psFound, $sProcName) = ();
  $sIQSrvCnt = 0;
  $sSwitch = $^O;    # Get OS..
  $psFound = 0;
  $sProcName = "$sSybIQBinDir/iqsrv15";
  $sGrepCmd = "/bin/grep";

  unless ( $hsCleanReasons { $iCleanReason } eq 'Offline_timed_out' ||
           $hsCleanReasons { $iCleanReason } eq 'Offline_was_ineffective' ) {

     if ( "linux" eq $sSwitch ) {
         $sPsCmd = "/bin/ps";
         $sPsCmdArgs = " -eo user,pid,args";
         $sPsCmd .= $sPsCmdArgs;

         if ( $sSybIQBinDir =~ /IQ-16_\d+\S*\/(bin|bin32|bin64)\b/ ){
            $sProcName = "$sSybIQBinDir/iqsrv16";
         }
         $sexecCmd = "$sPsCmd | $sGrepCmd $sProcName | $sGrepCmd $sServerName | $sGrepCmd -v grep";

         # Below command would be fired to see how many iqsrv15/iqsrv16 processess
         # running. Need to do this is because when only one iqsrv15/iqsrv16 server instance
         # is running with no database association we can use "stop_iq -stop one" command
         $sGetCntCmd = "$sPsCmd | $sGrepCmd $sProcName | $sGrepCmd -v grep";
     } elsif ( "solaris" eq $sSwitch ) {

         $sPsCmd = "/usr/ucb/ps";
         $sPsCmdArgs = " -auxwwl";
         $sPsCmd .= $sPsCmdArgs;

         if ( $sSybIQBinDir =~ /IQ-16_\d+\S*\/(bin|bin32|bin64)\b/ ){
              $sProcName = "$sSybIQBinDir/iqsrv16";
          }

         $sexecCmd = "$sPsCmd | $sGrepCmd $sProcName | $sGrepCmd $sServerName | $sGrepCmd -v grep";
         $sGetCntCmd = "$sPsCmd | $sGrepCmd $sProcName | $sGrepCmd -v grep";
     }

     my $sCommandArgsTemp = "";
     my $DecryptedPassword = DecryptPassword($sSybIQDBAdminPasswd);

     if($sSybIQDB){
        $sCommandArgs = " -c \"uid=$sSybIQDBAdmin;pwd=$DecryptedPassword;eng=$sServerName;links=tcpip\"";
        $sCommandArgsTemp = " -c \"uid=$sSybIQDBAdmin;pwd=XXXXX;eng=$sServerName;links=tcpip\"";
        &logTrace ( 20033,
        "<%s> Stopping SybaseIQ server instance [%s] with Command [%s] Args [%s]",
        $sMyName, $sServerName, $sCommand, $sCommandArgsTemp );
     }else{

        open(PIDCNT,"$sGetCntCmd |");
        while(<PIDCNT>){
          $sIQSrvCnt++;
        }

        if($sIQSrvCnt >= 1){
           open(PIDLIST,"$sexecCmd |");
           while(<PIDLIST>){
             chomp;
             @fields = split(/\s+/, $_);
             $psFound = 1;

             if(kill(0,$fields[1])){
                kill(15,$fields[1]);
                &logTrace ( 20034,
                "<%s> Stopping SybaseIQ server instance [%s].", $sMyName, $sServerName );
             }
             last;
           }
           if(!$psFound){
                &logTrace ( 20035,
                "<%s> No SybaseIQ server processes are left to be cleaned for instance [%s].", 
                $sMyName, $sServerName );
           }
        }
     }

    #-------------------
    # Try to be gentle..
    #-------------------
    if(!$psFound){
       &logInfo( 20036,
       "<%s> No SybaseIQ server processes are left to be cleaned for instance [%s].", 
       $sMyName, $sServerName );
    }else{
       &logInfo ( 20037,
       "<%s> Attempting to stop SybaseIQ with Command [%s] Args [%s]",
       $sMyName, $sCommand, $sCommandArgs );
    } 
    #------------------------
    # Request shutdown..
    #------------------------
    if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
      &logError ( 20038,
      "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
      $sMyName, abs ( $iTimeout ) );

      $iReturnCode = 0; # Failure..
    }
    elsif ( SybaseIQ::ExecLocalCmd ( $sSybIQUser, 
                                     $sEnvFile, 
                                     $sCommand, 
                                     $sCommandArgs, 
                                     \$iExitCode, 
                                     $iTimeout, 
                                     \@lsCmdout ) ) {
      &logTrace ( 20039, 
      "<%s> Cmd [%s] with args [%s] executed successfully",
      $sMyName, $sCommand, $sCommandArgs );
    }
  }

  #-----------------------------------------
  # Whack any persisting SybaseIQ processes..
  #-----------------------------------------
  &logTrace ( 20040,
  "<%s> Killing SybaseIQ processes for this instance", $sMyName );

  if ( SybaseIQ::KillSybaseIQProcesses ( $sServerName,
                                         $sStartOpts,
                                         $sSybIQBinDir,
                                         $sSybIQDB,
                                         $sSybIQDBConfFile,
                                         $sSybIQUser ) ) {
    &logTrace ( 20041,
    "<%s> Processes for this instance do not exist after shutdown",
    $sMyName );
  }
  else {
    &logError ( 20042,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );

    $iReturnCode = 0; # Failure..
  }
  
  
  return $iReturnCode;
}



=pod

 -------------------------------------------------------------------------
 Function : int GetResourceState ( IN String ServerName,
                                   IN String StartOpts,
                                   IN String SybIQBinDir,
                                   IN String SybIQDB,
                                   IN String SybIQDBConfFile,
                                   IN String SybIQUser );

 Description : First level monitor to determine state of resource.

 Parameters :

 $sServerName               Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $sStartOpts                Additional "Sybase IQ" server options while starting the SybaseIQ server instance.
 $sSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDB                  Complete path of the Sybase IQ database that is queried during detailed monitoring.
 $sSybIQDBConfFile          Complete path of the Sybase IQ database configuration file.
 $sSybIQUser                Specifies the user name required to run SybaseIQ commands.
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetResourceState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sServerName,
       $sStartOpts,
       $sSybIQBinDir,
       $sSybIQDB,
       $sSybIQDBConfFile,
       $sSybIQUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20043, "Subroutine <%s> called:
  ServerName               [%s]
  StartOpts                [%s]
  SybIQBinDir              [%s]
  SybIQDB                  [%s]
  SybIQDBConfFile          [%s]
  SybIQUser                [%s]",

  $sMyName, $sServerName,  $sStartOpts,  $sSybIQBinDir,  $sSybIQDB,
  $sSybIQDBConfFile, $sSybIQUser );
  

  my @liDummyPids = ();
  my $iReturnCode = SybaseIQ::GetProcState ( \@liDummyPids,
                                           $sServerName,
                                             $sStartOpts,
                                             $sSybIQBinDir,
                                             $sSybIQDB,
                                             $sSybIQDBConfFile,
                                             $sSybIQUser,
                                             1 ); # Use FFLM..

  if ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    &logTrace ( 20044,
    "<%s> Found all required processes. Return code set to [%s]",
    $sMyName, $iReturnCode );
  }
  else {
    &logTrace ( 20045,
    "<%s> Did not find all required processes. Returning [%s]",
    $sMyName, $iReturnCode );
  }

  #--------------------------------
  # Populate the global PID array..
  #--------------------------------
  @liRunningProcs = @liDummyPids;
                                                                                
  &logTrace ( 20046,
  "<%s> Populating global running proclist with [%s] PIDs [%s]",
  $sMyName, scalar @liRunningProcs, join ( ' ', @liRunningProcs ) );

  return ( $iReturnCode );
}


=pod

 -------------------------------------------------------------------------
 Function : int GetProcState ( OUT ListReference OnlinePids,
                               IN String ServerName,
                               IN String StartOpts,
                               IN String SybIQBinDir,
                               IN String SybIQDB,
                               IN String SybIQDBConfFile,
                               IN String SybIQUser,
                               IN Integer UseFFLM );

 Description : First level monitor to determine state of resource.

 Parameters :

 $rliOnlinePids             PIDs of relevant running processes
 $sServerName               Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $sStartOpts                Additional "Sybase IQ" server options while starting the SybaseIQ server instance.
 $sSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDB                  Complete path of the Sybase IQ database that is queried during detailed monitoring.
 $sSybIQDBConfFile          Complete path of the Sybase IQ database configuration file.
 $sSybIQUser                Specifies the user name required to run SybaseIQ commands.
 $iUseFFLM                  Filtering procedure
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetProcState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $rliOnlinePids,
       $sServerName,
       $sStartOpts,
       $sSybIQBinDir,
       $sSybIQDB,
       $sSybIQDBConfFile,
       $sSybIQUser,
       $iUseFFLM ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20047, "Subroutine <%s> called with:
  OnlinePids               [%s]
  ServerName               [%s]
  StartOpts                [%s]
  SybIQBinDir              [%s]
  SybIQDB                  [%s]
  SybIQDBConfFile          [%s]
  SybIQUser                [%s]
  UseFFLM                  [%s]",

  $sMyName,     $rliOnlinePids,     $sServerName,      $sStartOpts,
  $sSybIQBinDir,   $sSybIQDB,    $sSybIQDBConfFile,    $sSybIQUser,
  $iUseFFLM );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #----------------------
  # Initialise Variables..
  #----------------------
  my ( $iProcessCount, $iUseRFLM, $iFound, $iUid, $iPid, $sLang,
       $sKey, $rhlProc, $sPidFile, $sProc, $sTemp,
       @hProcListHash, @lsUpdates, @lsValues, %hiPids,
       %hiProcPids, %hsSrvrDetails, %hsCopy ) = ();

  @$rliOnlinePids = ();
  

  #------------------------------------
  # Attempt to get monitoring details..
  #------------------------------------
  if ( ( 0 == ( $iUid = getpwnam $sSybIQUser ) ) && # User is the super-user..
       ( 2 == $iUseFFLM ) ) {                  # And called to kill-by-UID..
    &logTrace ( 20048, "<%s> Unsafe to kill processes by User [%s] UID [%s].
    Using filter-based approach.", $sMyName, $sSybIQUser, $iUid ),
    $iUseFFLM = 1;
  }

  if ( $iUseFFLM != 2 ) { # Called for first level check..
    if ( SybaseIQ::GetSrvrDetails ( $sServerName, 
                                    $sStartOpts, 
                                    $sSybIQBinDir, 
                                    $sSybIQDB, 
                                    $sSybIQDBConfFile, 
                                    \%hsSrvrDetails ) ) {
      &logTrace ( 20049,
      "<%s> Got details for monitoring SybaseIQ instance", $sMyName );
  
      %hsCopy = %hsSrvrDetails;        # Backup..
      $iUseRFLM = scalar keys %hsCopy; # Set to entire count..
    }
    else {
      &logError ( 20050,
      "<%s> Failed to get details for monitoring SybaseIQ instance",
      $sMyName );
      return VCSDef_MonitorExit::UNKNOWN;
    }
                                  
    #---------------------------------
    # Check if we attempt to be fast.. 
    #---------------------------------
    if ( $iUseFFLM ) { # FFLM..
      $iUseRFLM = 0;   # Hopefully skip..

      foreach $sKey ( keys %hsCopy ) {          # Use copy..
        @lsUpdates = ();                        # Reset..
        @lsValues = @{ $hsCopy { $sKey } };     # Get values..
        $sTemp .= "\nSybaseIQ process [$sKey]\n"; # Display string..

        #----------------
        # Read in pairs..
        #----------------
        while ( ( $sPidFile, $sProc ) = splice ( @lsValues, 0, 2 ) ) {

          #-----------
          # Run FFLM..
          #-----------
          $iFound = SybaseIQ::FFLMStatus ( $sPidFile,
                                         $sProc,
                                         $iUid,
                                         \$iPid );

          #----------------
          # Update status..
          #----------------
          push ( @lsUpdates, $sPidFile, $sProc, $iFound );

          $sTemp .= "PID File     [$sPidFile]\n";
          $sTemp .= "Srch Process [$sProc]\n";
          $sTemp .= "FFLM Status  [$iFound]\n\n";
        }

        @{ $hsCopy { $sKey } } = @lsUpdates;

        #------------------------
        # Handle monitor status..
        #------------------------
        if ( VCSDef_MonitorExit::ONLINE == $iFound ) {
          push ( @$rliOnlinePids, $iPid ); # Push..
          
        }
        else {
          ++ $iUseRFLM; # Mark it for RFLM..
        }
      }

      #----------------
      # Print summary..
      #----------------
      &logTrace ( 20051,
      "<%s> SybaseIQ instance process details ---%s\n\n",
      $sMyName, $sTemp );

      ( $sTemp, $sPidFile, $sProc ) = (); # Release memory..
    }
    else { # Use RLFM..
      #-----------------------------------------
      # Mark status of each service as offline..
      #-----------------------------------------
      $iFound = VCSDef_MonitorExit::OFFLINE; # Set to offline..
      foreach $sKey ( keys %hsCopy ) {
        @lsUpdates = (); # Reset..

        @lsValues = @{ $hsCopy { $sKey } }; # Get values..
        while ( scalar @lsValues ) {
          push ( @lsUpdates, splice ( @lsValues, 0, 2 ), $iFound );
        }

        @{ $hsCopy { $sKey } } = @lsUpdates; # Update..
      }
    }
  }
  else { # Called for Kill..
    &logTrace ( 20052,
    "<%s> Called to get processes for User [%s] UID [%s]",
    $sMyName, $sSybIQUser, $iUid );
    ++ $iUseRFLM;
  }

  #-----------------------
  # Regular slow :-( FLM..
  #-----------------------
  if ( $iUseRFLM ) {
    &logTrace ( 20053,
    "<%s> Running regular first level monitor", $sMyName );

    #----------------------------------
    # Set LANG for better performance..
    #----------------------------------
    $sLang = $ENV { LANG }; # Backup..
    $ENV { LANG } = 'C';    # Now set..

    #-----------------------------------
    # Call processes and store in hash..
    #-----------------------------------
    $iProcessCount = Proc::GetLongProcessListHash ( \@hProcListHash );

    #----------
    # Restore..
    #----------
    defined $sLang ? $ENV { LANG } = $sLang : delete $ENV { LANG }; # Restore..

    #----------------------
    # Check if we got any..
    #----------------------
    if ( $iProcessCount ) {
      &logTrace ( 20054,
      "<%s> Got [%s] processes from proc table for filtering",
      $sMyName, $iProcessCount );
    }
    else {
      &logError ( 20055,
      "<%s> Call to get process list did not return any processes",
      $sMyName );
      return VCSDef_MonitorExit::OFFLINE; # Offline..
    }

    #-----------------------
    # Search for processes..
    #-----------------------
    foreach $rhlProc ( @hProcListHash ) {
      #--------------------------------------------------------------
      # Each $rhlProc is a ref. to a hash with the following keys..
      # "UID"     =>  <user id>
      # "PID"     =>  <process id>
      # "PPID"    =>  <parent process id>
      # "COMMAND" =>  <command string>
      #--------------------------------------------------------------

      next unless defined $$rhlProc { UID }; # Bug in Proc::GetLongProcessListHash..
      next if $$rhlProc { PID } == $$;       # Skip self..
      $iPid = $$rhlProc { PID };             # Assign..

#      &logTrace ( 20048, "<%s> Searching UID [%s] PID [%s] PPID [%s] CMD [%s]",
#      $sMyName, $$rhlProc { UID }, $$rhlProc { PID }, $$rhlProc { PPID },
#      $$rhlProc { COMMAND } );

      if ( $$rhlProc { UID } == $iUid ) { # Process owner..
        #-----------------------------------------------
        # Look for processes that were offline in FFLM..
        #-----------------------------------------------
        if ( $iUseFFLM != 2 ) { # Request to filter..
          foreach $sKey ( keys %hsCopy ) {
            @lsUpdates = (); # Reset..
            @lsValues = @{ $hsCopy { $sKey } }; # Get values..
    
            #--------------------------------------------------
            # Should have details per three elements of array..
            # <PIDFILE> <PROCESS> <STATE>
            #--------------------------------------------------
            while ( ( $sPidFile, $sProc, $iFound ) =
                      splice ( @lsValues, 0, 3 ) ) {
              if ( VCSDef_MonitorExit::OFFLINE == $iFound ) { # Earlier marked offline..
                if ( $$rhlProc { COMMAND } =~ /$sProc/ ) {    # Now found..
                  unshift ( @$rliOnlinePids, $iPid );         # Store..
    
                  &logTrace ( 20056,
                  "<%s> Found PID [%s] PPID [%s] CMD [%s] with filter [%s]",
                  $sMyName, $iPid, $$rhlProc { PPID }, $$rhlProc { COMMAND },
                  $sProc );

                  $hiProcPids { $sKey } = $iPid;        # Store..
                  
                  $iFound = VCSDef_MonitorExit::ONLINE; # Mark online..
                }
              }
    
              #----------------
              # Update status..
              #----------------
              push ( @lsUpdates, $sPidFile, $sProc, $iFound );
            }
    
            @{ $hsCopy { $sKey } } = @lsUpdates; # Update..
          }
        }
        else { # Request for kill..
          &logTrace ( 20057,
          "<%s> Pushing PID [%s] with PPID [%s] Cmd [%s] on stack",
          $sMyName, $iPid, $$rhlProc { PPID }, $$rhlProc { COMMAND } );
          push ( @$rliOnlinePids, $iPid ); # Store..
        } 
      }
      
    }
  }

  #------------------------------------
  # Set final status for each process..
  #------------------------------------
  my ( $iOnline, $iOffline, $sEntryPointName ) = ();

  ( $sEntryPointName = $0 ) =~ s#.*/##; # Basename..
  $sTemp = ''; # Ensure..

  foreach $sKey ( keys %hsCopy ) {
    @lsValues = @{ $hsCopy { $sKey } }; # Get values..
    $sTemp .= "\nProcess      [$sKey]\n";

    while ( ( $sPidFile, $sProc, $iFound ) =
              splice ( @lsValues, 0, 3 ) ) {
      #-----------------------------------
      # Create PID files for use in FFLM..
      #-----------------------------------
      if ( $iUseRFLM                                 &&      # In RFLM..
           $iUseFFLM != 2                            &&      # Asked to filter..
           $hiProcPids { $sKey }                     &&      # PID populated..
           ( $sEntryPointName =~ /^(monitor|online)$/o ) ) { # In monitor or online..

        &logTrace ( 20058,
        "<%s> Creating PID file [%s] with PID [%s]",
        $sMyName, $sPidFile, $hiProcPids { $sKey } );

        #---------
        # Create..
        #---------
        if ( SybaseIQ::CreatePidFile ( $sPidFile,
                                     $hiProcPids { $sKey } ) ) {
          &logTrace ( 20059,
          "<%s> Successfully created PID file [%s]", $sMyName, $sPidFile );
        }
        else {
          &logError ( 20060,
          "<%s> Could not create PID file [%s] successfully",
          $sMyName, $sPidFile );
        }
      }

      #-------------
      # Set status..
      #-------------
      if ( VCSDef_MonitorExit::ONLINE == $iFound ) {
        $iOnline = $iFound;
      }
      elsif ( VCSDef_MonitorExit::OFFLINE == $iFound ) {
        $iOffline = $iFound;
      }

      $sTemp .= "Srch Process [$sProc]\n";
      $sTemp .= "FLM Status   [$iFound]\n\n";
    }

    $sTemp .= "--------------------";
  }

  #-------------------
  # Set final status..
  #-------------------
  $iReturnCode = $iOffline || $iOnline || VCSDef_MonitorExit::UNKNOWN;

  #------------------------------------------------
  # Reset status based on PIDs, if not in monitor..
  #------------------------------------------------
  if ( $sEntryPointName ne 'monitor'                   &&
       ( VCSDef_MonitorExit::OFFLINE == $iReturnCode ) &&
       scalar @$rliOnlinePids ) {
    $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;

    &logTrace ( 20061,
    "<%s> Found [%s] PIDs [%s] for this instance of SybaseIQ.
    Resetting return code to [%s]",
    $sMyName, scalar @$rliOnlinePids, join ( ' ', @$rliOnlinePids ),
    $iReturnCode );
  }

  #-----------------------
  # Report on FLM status..
  #-----------------------
  if ( $iUseFFLM != 2 ) { # Request to filter..
    &logTrace ( 20062,
    "<%s> SybaseIQ process details ---\n%s\nReturn code is [%s]",
    $sMyName, $sTemp, $iReturnCode );
    $sTemp = undef; # Release..
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int Monitor ( IN String EnvFile,
                          IN String MonitorProgram,
                          IN Integer SecondLevelMonitor,
                          IN String ServerName,
                          IN String SybIQBinDir,
                          IN String SybIQDBAdmin,
                          IN String SybIQDBAdminPasswd,
                          IN String SybIQUser,
                          IN Integer $level_two );

 Description: Performs the in-depth monitoring

 Parameters :

 $sEnvFile                  Absolute path to the Sybase IQ environment file that will be sourced.
 $sMonitorProgram           Full path to external monitor program.
 $sServerName               Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $sSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDBAdmin             A Sybase database administrator who would has permission to stop Sybase IQ server instance.This user should have query privileges to the table in provided database.
 $sSybIQDBAdminPasswd       SybIQDBAdmPassowrd is the Sybase database password for SybIQDBAdmin who would query the Sybase database for detailed monitoring.
 $sSybIQUser                Specifies the user name required to run SybaseIQ commands.
 $level_two                Level Two Monitoring when IMF Enabled
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if in-depth monitoring cannot be completed.
 -------------------------------------------------------------------------

=cut

sub Monitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sEnvFile,
       $sMonitorProgram,
       $iSecondLevelMonitor,
       $sServerName,
       $sSybIQBinDir,
       $sSybIQDBAdmin,
       $sSybIQDBAdminPasswd,
       $sSybIQDB,
       $sSybIQUser,
       $level_two ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20063, "Subroutine <%s> called with:
  EnvFile                  [%s]
  MonitorProgram           [%s]
  SecondLevelMonitor       [%s] 
  ServerName               [%s]
  SybIQBinDir              [%s]
  SybIQDBAdmin             [%s]
  SybIQDBAdminPasswd       [XXXXX]
  SybIQDB                  [%s]
  SybIQUser                [%s]
  LevelTwoMonitor          [%s]",

  $sMyName, $sEnvFile, $sMonitorProgram, $iSecondLevelMonitor,
  $sServerName, $sSybIQBinDir, $sSybIQDBAdmin, $sSybIQDB, 
  $sSybIQUser, $level_two );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  my $iTillWhen = undef;

  #----------------------------------------------
  # In-depth monitoring, if correct time to run..
  #----------------------------------------------
  if ( -300 == $level_two ) { 	 
  $level_two = VCSagentFW::RunSecondLevelMonitor ( $iSecondLevelMonitor ); 	 
  &logTrace ( 20064, "<%s> Traditional second level monitoring, SLM flag for this monitor set to [%s] ", $sMyName, $level_two ); 	 
  }

  if ( $level_two >= 1 ) 
  {
    #--------------------------
    # Just In Time Validation..
    #--------------------------
    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode =
         SybaseIQ::DelayedAttributeCheck ( $sEnvFile,
                                           $sServerName,
                                           $sSybIQBinDir,
                                           $sSybIQDBAdmin,
                                           $sSybIQDBAdminPasswd,
                                           $sSybIQUser ) ) ) {
      &logTrace ( 20065,
      "<%s> Delayed attribute check completed successfully",
      $sMyName );
    }
    else {
      &logError ( 20066,
      "<%s> Attribute validation failed. Returning [%s]",
      $sMyName, $iReturnCode );
      return $iReturnCode;
    }

    &logTrace ( 20067,
    "<%s> Proceeding with second level monitoring", $sMyName );

    #-------------------------------------------------------
    # Upper bound..
    # Time in EPOCH when entry pt started + VCS time limit..
    #-------------------------------------------------------
    $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 );

    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode =
         SybaseIQ::SecondLevelMonitor ( $sEnvFile,
                                        $sServerName,
                                        $sSybIQBinDir,
                                        $sSybIQDBAdmin,
                                        $sSybIQDBAdminPasswd,
                                        $sSybIQDB,
                                        $sSybIQUser,
                                        $iTillWhen ) ) ) {

      &logTrace ( 20068,
      "<%s> SybaseIQ instance online with status [%s] after SLM",
      $sMyName, $iReturnCode );
    }
    else {
      &logError ( 20069,
      "<%s> SybaseIQ instance NOT online after SLM. Status [%s]",
      $sMyName, $iReturnCode );
      return $iReturnCode;
    }
  }

  #------------------------------------------------
  # Exit if external monitor has not been defined..
  #------------------------------------------------
  return $iReturnCode unless defined $sMonitorProgram;

  #---------------------------------------
  # MonitorProgram defined..
  # Separate the executable and the args..
  #---------------------------------------
  my ( $sExtCmd, $sExtCmdArgs ) = $sMonitorProgram # Separate
                                =~ /(\S+)\s*(.*)/; # Cmd and args..

  $sExtCmdArgs = '' unless defined $sExtCmdArgs;   # Prevent undefs..

  if ( $sExtCmd ) {                                # Defined..
    #----------------------------------------
    # Get time bearings, if not got earlier..
    #----------------------------------------
    $iTillWhen ||= $^T + VCSagentFW::GetEngineTimeout ( 1 );

    if ( VCSDef_MonitorExit::ONLINE ==             # and found online..
         ( $iReturnCode =                          # by the status..
         SybaseIQ::ExternalMonitor ( $sSybIQUser,
                                     $sEnvFile,
                                     $sExtCmd,
                                     $sExtCmdArgs,
                                     $iTillWhen ) ) ) {
      &logTrace ( 20070,
      "<%s> SybaseIQ instance found online with status [%s]
      after executing external monitor [%s] with args [%s]",
      $sMyName, $iReturnCode, $sExtCmd, $sExtCmdArgs );
    }
    else {
      &logError ( 20071,
      "<%s> SybaseIQ instance NOT online after executing external monitor
      [%s] with args [%s]. Status [%s]",
      $sMyName, $sExtCmd, $sExtCmdArgs, $iReturnCode );
    }
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int DelayedAttributeCheck ( IN String EnvFile,
                                        IN String ServerName,
                                        IN String SybIQBinDir,
                                        IN String SybIQDBAdmin,
                                        IN String SybIQDBAdminPasswd,
                                        IN String SybIQUser );

 Description : Just In Time Validation

 Parameters :

 $sEnvFile                  Absolute path to the Sybase IQ environment file that will be sourced.
 $sServerName               Specifies the logical name or the nickname which uniquely identifies i
                            the SybaseIQ server instance.
 $sSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDBAdmin             A Sybase database administrator who would has permission 
                            to stop Sybase IQ server instance.This user should have query 
                            privileges to the table in provided database.
 $sSybIQDBAdminPasswd       SybIQDBAdmPassowrd is the Sybase database password for SybIQDBAdmin 
                            who would query the Sybase database for detailed monitoring.
 $sSybIQUser                Specifies the user name required to run SybaseIQ commands.
 
 Returns  : Integer value as defined below:

 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub DelayedAttributeCheck {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sEnvFile,
       $sServerName,
       $sSybIQBinDir,
       $sSybIQDBAdmin,
       $sSybIQDBAdminPasswd,
       $sSybIQUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20072, "Subroutine <%s> called with:
  EnvFile                  [%s]
  ServerName               [%s]
  SybIQBinDir              [%s]
  SybIQDBAdmin             [%s]
  SybIQDBAdminPasswd       [XXXXX]
  SybIQUser                [%s]",

  $sMyName, $sEnvFile, $sServerName, $sSybIQBinDir, $sSybIQDBAdmin,
  $sSybIQUser );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my @lsArgs = qw ( EnvFile ServerName  SybIQBinDir  SybIQDBAdmin  SybIQDBAdminPasswd  SybIQUser );
  my @lsReqdArgs = qw ( EnvFile ServerName  SybIQBinDir  SybIQDBAdmin  SybIQDBAdminPasswd  SybIQUser );
  my ( @lsMissing, %hsArgList ) = ();

  @hsArgList { @lsArgs } = @_; # Assign..

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20073,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #--------------------
  # Return, if failed..
  #--------------------
  if ( VCSDef_MonitorExit::ONLINE != $iReturnCode ) {
    return $iReturnCode;
  }

  #----------------------------------
  # Validate parameter SybIQBinDir..
  # Since we have reached at a point where we have to do in-depth monitoring
  # we need to check if dbisql file is present which is needed for querying the
  # database.
  #----------------------------------
  my $sCmdDBISQL="";

  if ( ! -d $sSybIQBinDir ) {
    &logError ( 20074,
    "<%s> SybIQBinDir [%s] directory does not exist",
    $sMyName, $sSybIQBinDir );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  } elsif( $sSybIQBinDir =~ m/^(.*)\/.*\/.*/ ){
      $sCmdDBISQL = $1;
      $sCmdDBISQL .= "/DBISQL/bin/dbisql";
      if( ! -x $sCmdDBISQL ){
         $sCmdDBISQL = "";
         $sCmdDBISQL .=  $sSybIQBinDir."/dbisql";
         if( ! -x $sCmdDBISQL ){
            &logError ( 20075, "<%s> Cannot find dbisql program at location [%s] for indepth-monitoring",
            $sMyName, $sCmdDBISQL );
            $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
         }
      }
  }

  #------------------------------
  # Validate parameter EnvFile..
  #------------------------------
  if ( ! -f $sEnvFile ) {
    &logError ( 20076,
    "<%s> EnvFile [%s] is not a plain file",
    $sMyName, $sEnvFile );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #--------------------------------
  # Validate parameter SybIQUser..
  #--------------------------------
  if ( ! defined getpwnam $sSybIQUser ) {
    &logError ( 20077,
    "<%s> SybIQUser [%s] is not a valid UNIX login",
    $sMyName, $sSybIQUser );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int SecondLevelMonitor ( IN String EnvFile,
                                     IN String ServerName,
                                     IN String SybIQBinDir,
                                     IN String SybIQDBAdmin,
                                     IN String SybIQDBAdminPasswd,
                                     IN String SybIQUser,
                                     IN Integer TillWhen );

 Description : Does in-depth monitoring, if enabled.

 Parameters :

 $sEnvFile              Absolute path to the Sybase IQ environment file that will be sourced.
 $sServerName           Specifies the logical name or the nickname which uniquely 
                        identifies the SybaseIQ server instance.
 $sSybIQBinDir          The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDBAdmin         A Sybase database administrator who would has permission to stop Sybase 
                        IQ server instance.This user should have query privileges to the 
                        table in provided database.
 $sSybIQDBAdminPasswd   SybIQDBAdmPassowrd is the Sybase database password for SybIQDBAdmin 
                        who would query the Sybase database for detailed monitoring.
 $sSybIQUser            Specifies the user name required to run SybaseIQ commands.
 $iTillWhen             Time in EPOCH left for IInd level monitoring
 
 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if the command cannot be completed.
 -------------------------------------------------------------------------

=cut

sub SecondLevelMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sEnvFile,
       $sServerName,
       $sSybIQBinDir,
       $sSybIQDBAdmin,
       $sSybIQDBAdminPasswd,
       $sSybIQDB,
       $sSybIQUser,
       $iTillWhen ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20078, "Subroutine <%s> called with:
  EnvFile                  [%s]
  ServerName               [%s]
  SybIQBinDir              [%s]
  SybIQDBAdmin             [%s]
  SybIQDBAdminPasswd       [XXXXX]
  SybIQDB                  [%s]
  SybIQUser                [%s]
  TillWhen                 [%s]",

  $sMyName, $sEnvFile, $sServerName, $sSybIQBinDir, $sSybIQDBAdmin, $sSybIQDB,
  $sSybIQUser, $iTillWhen );  

  #------------------
  # Hopeful success..
  #------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $iTimeout, $iExitCode, $sCmdDBISQL, $sCommandArgs, @lsCmdout ) = ();
  
  #---------------------------------------------
  # Declare monitor command and args variables..
  #---------------------------------------------
  $sCmdDBISQL = "";
  if($sSybIQBinDir =~ m/^(.*)\/.*\/.*/ ){
      $sCmdDBISQL = $1;
      $sCmdDBISQL .= "/DBISQL/bin/dbisql";

      if( ! -x $sCmdDBISQL ){
         $sCmdDBISQL = "";
         $sCmdDBISQL .=  $sSybIQBinDir."/dbisql";
      }
  }
 
  my $decryptedSybIQDBAdminPasswd = DecryptPassword($sSybIQDBAdminPasswd);
  my $sCommandArgsTemp = "-c \"uid=$sSybIQDBAdmin;pwd=XXXXX;eng=$sServerName;links=tcpip\" -nogui \"describe SYSINFO\"";

  if($sSybIQDB =~ /\S+/){
     $sCommandArgs = "-c \"uid=$sSybIQDBAdmin;pwd=$decryptedSybIQDBAdminPasswd;eng=$sServerName;links=tcpip;dbf=$sSybIQDB\" -nogui \"describe SYSINFO\"";
  } else {
     &logError ( 20161, "SybaseIQ database name missing. In-depth monitoring requires a database name");
     return VCSDef_MonitorExit::UNKNOWN;
  }

  #-------------------------------
  # Validate and execute command..
  #-------------------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20079,
    "<%s> Insufficient time to continue in-depth monitoring.
    Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  elsif ( SybaseIQ::ExecLocalCmd ( $sSybIQUser, 
                                   $sEnvFile, 
                                   $sCmdDBISQL, 
                                   $sCommandArgs, 
                                   \$iExitCode, 
                                   $iTimeout, 
                                   \@lsCmdout ) ) {
    &logTrace ( 20080,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCmdDBISQL, $sCommandArgsTemp );
    
  }
  elsif ( ! defined $iExitCode ) {
    $lsCmdout [0] ||= 'undef'; # Set if not defined..
    $iReturnCode = $lsCmdout [0] eq 'timeout'? VCSDef_MonitorExit::TIMEOUT: # Timeout..
                                               VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  else {
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int ExternalMonitor ( IN String SybIQUser,
                                  IN String EnvFile,
                                  IN String Cmd,
                                  IN String CmdArgs,
                                  IN Integer TillWhen );

 Description : Handles user defined external monitor program

 Parameters :

 $sSybIQUser                Specifies the user name required to run SybaseIQ commands.
 $sEnvFile                  Absolute path to the Sybase IQ environment file that will be sourced.
 $sCmd                      User defined utilty to execute
 $sCmdArgs                  Command line arguments
 $iTillWhen                 Time in EPOCH left for IInd level monitoring
 
 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if the command cannot be completed.
 -------------------------------------------------------------------------

=cut

sub ExternalMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sSybIQUser,
       $sEnvFile,
       $sCmd,
       $sCmdArgs,
       $iTillWhen ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20081, "Subroutine <%s> called with:
  SybIQUser                [%s]
  EnvFile                  [%s]
  Cmd                      [%s]
  CmdArgs                  [%s]
  TillWhen                 [%s]",

  $sMyName, $sSybIQUser, $sEnvFile, $sCmd, $sCmdArgs, $iTillWhen );

  #------------------
  # Hopeful success..
  #------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #------------------------
  # Setup local variables..
  #------------------------
  my ( $iExitCode, $iTimeout, @lsCmdout ) = ();
  my $iBuf = 5; # Secs..

  #-------------------------
  # Handled exit code hash..
  #-------------------------
  my %hiExitCodes = (  0, VCSDef_MonitorExit::ONLINE,
                     110, VCSDef_MonitorExit::ONLINE,
                       1, VCSDef_MonitorExit::OFFLINE,
                     100, VCSDef_MonitorExit::OFFLINE,
                          VCSDef_MonitorExit::TIMEOUT,
                          VCSDef_MonitorExit::TIMEOUT );

  #---------------------------------------------
  # Check if command can be found and executed..
  #---------------------------------------------
  if ( ! -e $sCmd ) {
   &logError ( 20082,
    "<%s> External monitor [%s] does not exist",
    $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( ! -x $sCmd ) {
   &logError ( 20083,
    "<%s> External monitor [%s] not executable.",
    $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( $sEnvFile =~ /\S/ && ! -e $sEnvFile ) {
    &logError ( 20084,
    "<%s> EnvFile [%s] does not exist", $sMyName, $sEnvFile );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
elsif ( ( $iTimeout = $iTillWhen - ( time + $iBuf ) ) <= 5 ) { # Lack of time..
   &logError ( 20085,
    "<%s> Insufficient time to proceed with external monitor [%s].
    Short by [%s] sec", $sMyName, $sCmd, abs ( $iTimeout ) );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }

  #--------------------------------------------
  # Return if validation failed at this point..
  #--------------------------------------------
  unless ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    return $iReturnCode;
  }

  &logTrace ( 20086,
  "<%s> Executing external monitor with Command [%s] Args [%s] in [%s] secs",
  $sMyName, $sCmd, $sCmdArgs, $iTimeout );

  #----------------------
  # Execute the command..
  #----------------------
  SybaseIQ::ExecLocalCmd ( $sSybIQUser, 
                           $sEnvFile, 
                           $sCmd, 
                           $sCmdArgs, 
                           \$iExitCode, 
                           $iTimeout, 
                           \@lsCmdout );
                               
  #------------------------
  # Report on exit status..
  #------------------------
  if ( ! defined $iExitCode ) {
    $lsCmdout [0] ||= 'undef'; # Set if not defined..
    $iExitCode = $lsCmdout [0] eq 'timeout'? VCSDef_MonitorExit::TIMEOUT: # Timeout..
                                             VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  elsif ( $hiExitCodes { $iExitCode } &&
          VCSDef_MonitorExit::ONLINE ==
          $hiExitCodes { $iExitCode } ) {
    &logTrace ( 20087,
    "<%s> Command [%s] with args [%s] executed successfully with exit code [%s].
    Cmd output [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode, join ( chr 012, @lsCmdout ) );
  }
  else {
    &logWarn ( 20088,
    "<%s> Command [%s] with args [%s] execution failed with exit code [%s].
    Cmd output [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode, join ( chr 012, @lsCmdout ) );
  }

  #--------------------------------
  # Setup appropriate return code..
  #--------------------------------
  $iReturnCode = $hiExitCodes { $iExitCode } ||
                 VCSDef_MonitorExit::UNKNOWN;

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool CreatePidFile ( IN String PidFile,
                                 In Integer Pid );


 Description : Creates PID file with given PID

 Parameters :

 $sPidFile     File to create
 $iPid         PID to store

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub CreatePidFile {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sPidFile,
       $iPid ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20089, "Subroutine <%s> called with:
  PidFile         [%s]
  Pid             [%s]",

  $sMyName, $sPidFile, $iPid );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #-----------------------
  # Check against undefs..
  #-----------------------
  unless ( defined $iPid && $iPid =~ /^\d+$/o) {
    $iPid ||= 'undef'; # Prevent undefs.
    &logError ( 20090, "<%s> PID [%s] is not a valid positive number",
    $sMyName, $iPid );
    return 0; # Proceed no further..
  }

  #------------------
  # Create PID file..
  #------------------
  &logTrace ( 20091,
  "<%s> Creating PID file [%s] with PID [%s]",
  $sMyName, $sPidFile, $iPid );

  if ( sysopen ( PIDFILE, $sPidFile, O_WRONLY|O_SYNC|O_CREAT|O_TRUNC,
                 0600 ) ) {
    print PIDFILE $iPid, "\n"; # Update..

    unless ( close ( PIDFILE ) ) {
      &logWarn ( 20092,
      "<%s> Failed to close PidFile [%s] after update: [%s]",
      $sMyName, $sPidFile, $! );
      $iReturnCode = 0;
    }
  }
  else {
    &logWarn ( 20093,
    "<%s> Failed to create PID file [%s] with PID [%s]",
    $sMyName, $sPidFile, $iPid );
    $iReturnCode = 0;
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int FFLMStatus ( IN String PidFile,
                             IN String Proc,
                             IN Integer Uid,
                             OUT RefInt Pid );

 Description: Checks status of Process listed in PidFile, if running

 Parameters :

 $sPidFile       Absolute path to PID file of service
 $sProc          Process string to search for in proc table
 $iUid           UNIX UID of owner
 $riPid          Reference to existing PID

 Returns : Integer value as defined below:
 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 -------------------------------------------------------------------------

=cut

sub FFLMStatus {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sPidFile,
       $sProc,
       $iUid,
       $riPid ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20094, "Subroutine <%s> called with:
  PidFile                [%s]
  Proc                   [%s]
  UID                    [%s]
  riPid                  [%s]",

  $sMyName, $sPidFile, $sProc, $iUid, $riPid );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..
  $$riPid = 0; # Reset..

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $iPid, $rhlProc, @lProcessHash ) = ();

  #----------------------------------
  # Start by testing the attributes..
  #----------------------------------
  if ( $sProc !~ /\S/ ) {
    &logError ( 20095, "<%s> Called with empty [%s] process string",
    $sMyName, $sProc );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }
  elsif ( ! -e $sPidFile ) {
    &logTrace ( 20096, "<%s> PID file [%s] does not exist",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }
  elsif ( ! -s $sPidFile ) {
    &logWarn ( 20097, "<%s> PID file [%s] is of zero bytes",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  #---------------------
  # Return, if offline..
  #---------------------
  return $iReturnCode unless VCSDef_MonitorExit::ONLINE == $iReturnCode;
 
  if ( open ( PIDFILE, $sPidFile ) ) {
    ( $iPid ) = grep { ( s/\n// || 1 ) && # Chomp..
                         s/.*?(\d+)$/$1/  # Get PID..
                } <PIDFILE>;

    $iPid = '' unless defined $iPid;      # Avoid undefs..

    close ( PIDFILE ) ||                  # Close file..
    &logWarn ( 20098,
    "<%s> Could not close PID file [%s] after read [%s]",
    $sMyName, $sPidFile, $! );
  }
  else {
    &logError ( 20099,
    "<%s> Failed to open PID file [%s] for read: [%s]",
    $sMyName, $sPidFile, $! );
    return VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  #------------------------
  # Check if PID is valid..
  #------------------------
  if ( $iPid =~ /\d+/ &&     # If the valid number..
       kill ( 0, $iPid ) ) { # is a PID that's alive..

    &logTrace ( 20100,
    "<%s> PID [%s] from PID file [%s] exists",
    $sMyName, $iPid, $sPidFile );

    #-----------------------
    # Search in proc table..
    #-----------------------
    if ( SybaseIQ::GetPidDetails ( $iPid,
                                 \@lProcessHash ) ) {
      #--------------
      # Got details..
      #--------------
      foreach $rhlProc ( @lProcessHash ) {
        &logTrace ( 20101,
        "<%s> Process details: UID [%s] PID [%s] PPID [%s] ARGS [%s]",
        $sMyName, $$rhlProc { UID }, $$rhlProc { PID }, $$rhlProc { PPID },
        $$rhlProc { COMMAND } );

        #--------------
        # Now compare..
        #--------------
        if ( ( $$rhlProc { UID } == $iUid ) &&       # Owner match..
               $$rhlProc { COMMAND } =~ /$sProc/ ) { # Filter match..
          &logTrace ( 20102,
          "<%s> PID [%s] owned by UID [%s] with ARGS [%s] contains [%s]",
          $sMyName, $iPid, $$rhlProc { UID }, $$rhlProc { COMMAND }, $sProc );
          $$riPid = $iPid; # Store..
        }
        else {
          &logTrace ( 20103,
          "<%s> Ignoring PID [%s] owned by UID [%s] with ARGS [%s] with string [%s]",
          $sMyName, $iPid, $$rhlProc { UID }, $$rhlProc { COMMAND }, $sProc );
          $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
        }
      }
    }
    else {
      &logTrace ( 20104,
      "<%s> Could not get details for PID [%s]",
      $sMyName, $iPid );
      $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
    }
  }
  else {
    &logTrace ( 20105, "<%s> PID [$iPid] does not exist",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  @lProcessHash = (); # Truncate..
  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool KillSybaseIQProcesses ( IN String ServerName,
                                         IN String StartOpts,
                                         IN String SybIQBinDir,
                                         IN String SybIQDB,
                                         IN String SybIQDBConfFile,
                                         IN String SybIQUser );

 Description  : Kills processes for this instance of SybaseIQ
 
 Parameters :

 $sServerName             Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $sStartOpts              Additional "Sybase IQ" server options while starting the SybaseIQ server instance.
 $sSybIQBinDir            The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDB                Complete path of the Sybase IQ database that is queried during detailed monitoring.
 $sSybIQDBConfFile        Complete path of the Sybase IQ database configuration file.
 $sSybIQUser              Specifies the user name required to run SybaseIQ commands.
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------
=cut

sub KillSybaseIQProcesses {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sServerName,
       $sStartOpts,
       $sSybIQBinDir,
       $sSybIQDB,
       $sSybIQDBConfFile,
       $sSybIQUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20106, "Subroutine <%s> called with:
  ServerName               [%s]
  StartOpts                [%s]
  SybIQBinDir              [%s]
  SybIQDB                  [%s]
  SybIQDBConfFile          [%s]
  SybIQUser                [%s]",

  $sMyName, $sServerName,  $sStartOpts,  $sSybIQBinDir,  $sSybIQDB,
  $sSybIQDBConfFile, $sSybIQUser );
 
  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  #-------------------------
  # Set up local variables..
  #-------------------------
  my ( @liGoners, @liUnDead ) = ();

  #---------------------------------------------
  # Get list of processes running for this Uid..
  #---------------------------------------------
  if ( ( caller 1 )[3] eq 'SybaseIQ::Start' ) { # During online..
    @liGoners = @liRunningProcs;              # Get PIDs to kill..
  }
  else {
    SybaseIQ::GetProcState ( \@liGoners,
                           $sServerName,
                             $sStartOpts,
                             $sSybIQBinDir,
                             $sSybIQDB,
                             $sSybIQDBConfFile,
                             $sSybIQUser,
                             0 ); # Set filtering choice..
  }

  #--------------------------
  # Sort in ascending order..
  #--------------------------
  @liGoners = sort { $a <=> $b } @liGoners;

  #------------------------------------------------------------------
  # We do a two level kill. The SIGINQ ensures  that  the  PID  is  a
  # valid process before we actually issue the SIGKILL. We don't want
  # to report a failure for SIGKILL in case  of  defunct  or  missing
  # processes.
  #------------------------------------------------------------------
  if ( scalar @liGoners ) {
    #----------------------
    # Failed to kill some..
    #----------------------
    if ( @liUnDead = grep {
                         kill ( 0, $_ ) && # Valid..
                       ! kill ( 9, $_ )    # and could NOT be killed..
                     } @liGoners ) {       # From what's found..
      &logError ( 20107,
      "<%s> PIDs not killed [%s] Total not killed [%s]",
      $sMyName, join ( ' ', @liUnDead ), scalar @liUnDead );
      $iReturnCode = 0; # Failure..
    }
    else { # All killed..
      &logInfo ( 20108,
      "<%s> Killed PIDs [%s] Total killed: [%s]",
      $sMyName, join ( ' ', @liGoners ), scalar @liGoners );
    }
  }
  else {
    &logTrace ( 20109,
    "<%s> Could not locate any process this instance of SybaseIQ",
    $sMyName );
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool ExecLocalCmd ( IN String User,
                                IN String EnvFile,
                                IN String Command,
                                IN String CommandArgs,
                                OUT RefInt ExitCode,
                                IN Integer Timeout,
                                OUT ListReference Cmdout );

 Description : Wrapper function for
               Sys::RunWithEnvCmdWithOutputWithTimeOut to collect output
               of the command.
 
 Parameters :

 $sUser            UNIX user to run the command as
 $sEnvFile         Absolute path to location of EnvFile to source
 $sCommand         Absolute path to location of command
 $sCommandArgs     Arguments to command
 $riExitCode       Reference to collect exit code of command
 $iTimeout         Time limit to execute the command
 $rlsCmdout        Stores o/p from command if successful
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ExecLocalCmd {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sUser,
       $sEnvFile,
       $sCommand,
       $sCommandArgs,
       $riExitCode,
       $iTimeout,
       $rlsCmdout ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  my $sDisplayArgs = $sCommandArgs;
  my $passStr = undef;

  if($sCommand =~ /.*dbisql$/){
     $sDisplayArgs =~ s/pwd=(\w+);/pwd=XXXXX;/;
     $passStr = $1;
  }

  if($sCommand =~ /.*dbstop$/){
     $sDisplayArgs =~ s/pwd=(\w+);/pwd=XXXXX;/;
     $passStr = $1; 
  }

  &logTrace ( 20110, "Subroutine <%s> called with:
  User        [%s]
  EnvFile     [%s]
  Command     [%s]
  CommandArgs [%s]
  riExitCode  [%s]
  Timeout     [%s]
  rlsCmdout   [%s]",

  $sMyName,
  $sUser, $sEnvFile, $sCommand, $sDisplayArgs, $riExitCode,
  $iTimeout, $rlsCmdout );

  #---------------------------------------
  # Setup return code to hopeful success..
  #---------------------------------------
  my $iReturnCode = 1;

  

  #-------------------
  # Set up arguments..
  #-------------------
  my $bSourceUserEnv = 0; # Do not source..
  my $bTimedOut = 0;      # Timeout variable..

  @$rlsCmdout = (); # Reset..

  #-----------
  # Validate..
  #-----------
  if ( ! -e $sCommand ) {
    &logError ( 20111,
    "<%s> Cmd [%s] does not exist", $sMyName, $sCommand );
    $iReturnCode = 0; # Failure..
  }
  elsif ( ! -x $sCommand ) {
    &logError ( 20112,
    "<%s> Cmd [%s] is not executable", $sMyName, $sCommand );
    $iReturnCode = 0; # Failure..
  }
  elsif ( $iTimeout <= 1 ) {
    &logError ( 20113,
    "<%s> Insufficient time [%s] to execute Cmd [%s] with Args [%s].",
    $sMyName, $iTimeout, $sCommand, $sDisplayArgs );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return if failed.. 

  #----------------------
  # Set LOGNAME to user..
  #----------------------
  my $sOldLogName = $ENV { LOGNAME }; # Backup..
  $ENV { LOGNAME } = $sUser;          # Now set..

  &logTrace ( 20114,
  "<%s> Executing Command [%s] with Args [%s]",
  $sMyName, $sCommand, $sDisplayArgs );

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut ( $sEnvFile,
                                                           $bSourceUserEnv,
                                                           $sCommand,
                                                           $sCommandArgs,
                                                           $riExitCode,
                                                           $sUser,
                                                           $rlsCmdout,
                                                           \$bTimedOut,
                                                           $iTimeout,
                                                           "","",$passStr);

  #---------------------------
  # Remove trailing newlines..
  #---------------------------
  chomp @$rlsCmdout;

  #--------------------
  # Check return code..
  #--------------------
  if ( $iReturnCode ) {   # No error with command execution..
    if ( $$riExitCode ) { # Failure..
      &logTrace ( 20115,
      "<%s> Command [%s] with args [%s] returned exit code [%s]",
      $sMyName, $sCommand, $sDisplayArgs, $$riExitCode );
      $iReturnCode = 0; # Failure..
    }
    else {
      &logTrace ( 20116,
      "<%s> Command [%s] with args [%s] executed successfully with exit code [%s]",
      $sMyName, $sCommand, $sDisplayArgs, $$riExitCode );
    }
  }
  else { # Execution error..
    @$rlsCmdout = (); # Reset..

    if ( $bTimedOut ) {  # We timed out..
      &logError ( 20117,
      "<%s> Command [%s] with args [%s] did not complete within [%s] seconds",
      $sMyName, $sCommand, $sDisplayArgs, $iTimeout );
      $$rlsCmdout [0] = 'timeout'; # Timeout..
    }
    else { # Failure due to something else..
      &logError ( 20118,
      "<%s> Failed in call to RunWithEnvCmdWithOutputWithTimeOut",
      $sMyName );
    }
  }

  #----------
  # Restore..
  #----------
  defined $sOldLogName ? $ENV { LOGNAME } = $sOldLogName : delete $ENV { LOGNAME };

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool GetPidDetails ( IN Integer Pid,
                                 OUT RefList ProcList );


 Description : Gets long process details for a known Pid if running

 Parameters :

 $iPid         PID whose details are requested
 $rlProcList   Ref to List to store details
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub GetPidDetails {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iPid,
       $rlProcList ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20119, "Subroutine <%s> called with:
  Pid             [%s]
  rlProcList      [%s]",

  $sMyName, $iPid, $rlProcList );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #---------------
  # Validate PID..
  #---------------
  unless ( defined $iPid    &&   # Catch undefs..
           $iPid =~ /^\d+$/ &&   # And invalid PIDs..
           kill ( 0, $iPid ) ) { # And defunct ones..
    &logError ( 20120,
    "<%s> PID [%s] is invalid", $sMyName, $iPid );
    return 0;
  }

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sPsCmd, $sPsCmdArgs, $sLang, $sFmt, $sUnix95, $sSwitch, $iExitCode,
       %hsProc ) = ();

  $sSwitch = $^O;    # Get OS..
  @$rlProcList = (); # Truncate..

  #-------------
  # Store LANG..
  #-------------
  $sLang = $ENV { LANG };
  $ENV { LANG } = 'C';

  #---------------------------
  # Switch to the correct OS..
  #---------------------------
  if ( "solaris" eq $sSwitch ) {
      $sPsCmd = "/usr/ucb/ps";
      $sPsCmdArgs = "axwwl $iPid";
      $sFmt = '[\da-z]+\s+([\d]+)\s+([\d]+)\s+([\d]+)\s+.*?[\d]:[\d]{2}\s(.*)';
  }
  elsif ( "hpux" eq $sSwitch ) {
      #-----------------------
      # Force XPG4 behaviour..
      #-----------------------
      $sUnix95 = $ENV { UNIX95 };
      $ENV { UNIX95 } = 'XPG4';

      $sPsCmd = "/usr/bin/ps";
      $sPsCmdArgs = "-p $iPid -xxlo uid,pid,ppid,args";
      $sFmt = '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)';
  }
  elsif ( "aix" eq $sSwitch ) {
      $sPsCmd = "/usr/bin/ps";
      $sPsCmdArgs = "axwwl $iPid";
      $sFmt =  '[\da-e]+\s+[A-Z]\s+([\d]+)\s+([\d]+)\s+([\d]+)\s+.*?[\d]:[\d]{2}\s(.*)';
  }
  elsif ( "linux" eq $sSwitch ) {
      $sPsCmd = "/bin/ps";
      $sPsCmdArgs = "wwo uid,pid,ppid,args $iPid";
      $sFmt = '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)';
  }

  &logTrace ( 20121, "<%s> OS [%s] PsCmd [%s] PsCmdArgs [%s]",
  $sMyName, $sSwitch, $sPsCmd, $sPsCmdArgs );

  #-------------------
  # Validate command..
  #-------------------
  if ( ! -x $sPsCmd ) {
    &logError ( 20122,
    "<%s> ps(1b) command [%s] is not executable", $sMyName, $sPsCmd );
    return 0; # Failure..
  }

  #-------------------------------
  # Fire command and get details..
  #-------------------------------
  @hsProc { qw ( UID PID PPID COMMAND ) } =
  map {
    s/\n//;                             # Chomp..
    s/^\s+//;                           # Strip leading spaces..
    /^$sFmt$/o;                         # Extract fields..
  } readpipe ( "$sPsCmd $sPsCmdArgs" ); # From the command..

  $iExitCode = $? >> 8; # Get exit code..

  #------------------------
  # Restore env variables..
  #------------------------
  defined $sLang ? $ENV { LANG } = $sLang : delete $ENV { LANG };
  defined $sUnix95 ? $ENV { UNIX95 } = $sUnix95 : delete $ENV { UNIX95 };

  #------------------
  # Check exit code..
  #------------------
  if ( $iExitCode ) {
    &logError ( 20123,
    "<%s> Cmd [%s] with args [%s] failed with exit code [%s]",
    $sMyName, $sPsCmd, $sPsCmdArgs, $iExitCode );

    %hsProc = (); # Truncate..
    return 0;
  }

  #---------------------
  # Fill up the bucket..
  #---------------------
  push ( @$rlProcList, \%hsProc ); # Update..

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool GetSrvrDetails ( IN String ServerName,
                                  IN String StartOpts,
                                  IN String SybIQBinDir,
                                  IN String SybIQDB,
                                  IN String SybIQDBConfFile,
                                  OUT RefHashList SrvrDetails );

 Description : Gives PID file and process string for given server type

 Parameters  :

 $sServerName               Specifies the logical name or the nickname which uniquely identifies the SybaseIQ server instance.
 $sStartOpts                Additional "Sybase IQ" server options while starting the SybaseIQ server instance.
 $sSybIQBinDir              The directory where SybaseIQ binary "start_iq" exist.
 $sSybIQDB                  Complete path of the Sybase IQ database that is queried during detailed monitoring.
 $sSybIQDBConfFile          Complete path of the Sybase IQ database configuration file.
 $rhlSrvrDetails            Stores details of PID file and process string
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub GetSrvrDetails {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sServerName,
       $sStartOpts,
       $sSybIQBinDir,
       $sSybIQDB,
       $sSybIQDBConfFile,
       $rhlSrvrDetails ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20124, "Subroutine <%s> called with:
  ServerName               [%s]
  StartOpts                [%s]
  SybIQBinDir              [%s]
  SybIQDB                  [%s]
  SybIQDBConfFile          [%s]
  SrvrDetails              [%s]",

  $sMyName, $sServerName,  $sStartOpts,  $sSybIQBinDir,  $sSybIQDB,
  $sSybIQDBConfFile, $rhlSrvrDetails );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = 1; # Set to true..

  #-------------------
  # Define variables..
  #-------------------
  my ( $sKey, $sProc, $sPidFile, $sPIDDir, $sResourceType,
       $sResourceName, $iPat ) = ();

  #-------------------------------------
  # List of search patterns to monitor..
  #-------------------------------------
  my @lsSrchPatterns = ( qr@$sSybIQBinDir/iqsrv1(5|6)\s*$sStartOpts -n $sServerName\s+@ );

  #----------------------------------------------
  # Derive our resource type and resource names..
  #----------------------------------------------
  $sResourceType = ( $0 =~ m@([^/]+)@g ) [-2];
  $sResourceName   = $ENV { VCS_LOG_RESOURCE_NAME };
  $sPIDDir = "/tmp/.VRTS$sResourceType/$sResourceName/PID";

  #------------------------------
  # Create PID folder if needed..
  #------------------------------
  if ( -d $sPIDDir ) {
    &logTrace ( 20125, "<%s> PID folder [%s] exists",
    $sMyName, $sPIDDir );
  }
  elsif ( ! mkpath ( $sPIDDir, 0, 0700 ) ) {
    &logWarn ( 20126, "<%s> Failed to create PID folder [%s]",
    $sMyName, $sPIDDir );
    return 0; # Cannot proceed..
  }

  %$rhlSrvrDetails = (); # Truncate..

  #-----------------------------------------------
  # Set appropriate filter strings and PID files..
  #-----------------------------------------------
  foreach $sProc ( @lsSrchPatterns ) {
    $sKey = 'srchstr_' . ++ $iPat;
    $sPidFile = "$sPIDDir/pidfile.$sKey";
    @{ $$rhlSrvrDetails { $sKey } } = ( $sPidFile, $sProc ); # Store..
  }

  return $iReturnCode;
}

=pod

----------------------------------------------------------------------------
Function:  String DecryptPassword ( IN String CryptedPassword)

Decrypts the password using VCS decrypt subroutine.

----------------------------------------------------------------------------

=cut

sub DecryptPassword
{
  my ($sCryptedPassword ) = @_;
  my ($sCmdDecrypt, $sDecryptedAdminPassword, $iExitCode) = ();

  my $sMyName = ( caller 0 )[3];
  &logTrace(20127,"Subroutine <%s> called.", $sMyName );

  #-------------------------
  # Get decrypted password..
  #-------------------------
  if ( defined $sCryptedPassword) {
    #-----------
    # Validate..
    #-----------
    if ( ! -x $sCmdVcsDecrypt ) {
      &logError ( 20162,
      "<%s> Command [%s] is not executable", $sMyName, $sCmdVcsDecrypt );
      return 0; # Failure..
    }

    $sCmdDecrypt = $sCmdVcsDecrypt;                       # Get global..
    ( $sDecryptedAdminPassword ) = grep {                 # Get decrypted..
                          s/([^\w-])/\\$1/g || 1          # Escape non-words..
                        } readpipe ( "$sCmdDecrypt $sCryptedPassword" );

    #--------------------
    # Collect exit code..
    #--------------------
    if ( $iExitCode = $? >> 8 ) {
      &logError ( 20163, "<%s> Failed to decrypt crypted string [%s]",$sMyName, $sCryptedPassword);
      return 0; # Failure..
    } else {
      return $sDecryptedAdminPassword;
    }
  }
}

1; # So PERL knows its included..
